# $Id: Response.pm,v 1.5 2006/12/29 20:00:37 koba Exp $
#	Copyright(c) 2006 Satoshi Kobayashi, All rights reserved.

package XiuMai::HTTP::Response;

use strict;
use XiuMai;
use XiuMai::XHTML;
use XiuMai::Locale;
use XiuMai::Auth;
use XiuMai::Util qw(cdata url_decode rfc1123_date canonpath randstr);
use vars qw($VERSION %STATUS_LINE);

$VERSION = $XiuMai::VERSION;

%STATUS_LINE = (

	200	=> 'OK',
	201	=> 'Created',

	301 => 'Moved Permanently',
	302 => 'Found',
	303 => 'See Other',

	400	=> 'Bad Request',
	401	=> 'Unauthorized',
	403	=> 'Forbidden',
	404	=> 'Not Found',
	405	=> 'Method Not Allowed',
);

sub new {
	my $class = shift;
	my ($req) = @_;

	my $self = {};
	$self->{Request} = $req;
	$self->{CGI} = $req->{CGI};
	$self->{CGI}->charset('UTF-8');

	return bless $self, $class;
}

sub session_cookie {
	my $self = shift;

	my $cookie = $self->{CGI}->cookie('XIUMAI_SESSION') || randstr(32);
	my $path = $self->{Request}->xiumai_base_url;

	return $self->{CGI}->cookie(
				-name	=> 'XIUMAI_SESSION',
				-value	=> $cookie,
				-path	=> $path,
			);
}

sub auth_cookie {
	my $self = shift;

	return $self->{auth_cookie}		if (defined $self->{auth_cookie});

	my $cookie  = $self->{CGI}->cookie('XIUMAI_AUTH')	or return;
	my $path    = $self->{Request}->xiumai_base_url;
	my $expdate = (defined $self->{Request}->xiumai_user)
								? qq(+${XiuMai::Auth::EXPDATE}d) : '-1d';

	return $self->{CGI}->cookie(
				-name	 => 'XIUMAI_AUTH',
				-value	 => $cookie,
				-path	 => $path,
				-expires => $expdate,
			);
}

sub header {
	my $self = shift;
	my %param = @_ == 1 ? ( -type => shift @_ ) : @_;

#	$param{-type}			= 'application/xhtml+xml'
#											if (! exists $param{-type});
	$param{-cookie}			= [ $self->session_cookie, $self->auth_cookie ];
	$param{-last_modified}	= rfc1123_date($param{-last_modified})
											if (exists $param{-last_modified});
	$param{-nph}			= $ENV{MOD_PERL}	if (exists $param{-status});

	return $self->{CGI}->header(%param);
}

sub redirect {
	my $self = shift;
	my %param = @_ == 1 ? ( -uri => shift @_ ) : @_;

	$param{-cookie}	= [ $self->session_cookie, $self->auth_cookie ];
	$param{-nph}	= $ENV{MOD_PERL};
	$param{-type}   = 'text/html';

	return $self->{CGI}->redirect(%param);
}

sub print {
	my $self = shift;
	my ($r) = @_;

	my %param;
	$param{-type}			= $r->type	if (defined $r->type);
	$param{-content_length}	= $r->size	if (defined $r->size);
	$param{-last_modified}	= $r->mtime	if (defined $r->mtime);
	$param{-content_language}	= join(', ', $r->lang)	if ($r->lang);
#	$param{-cache_control}	= 'private';

	print $self->header(%param);
	$r->print					if ($self->{Request}->method eq 'GET');
	exit;
}

sub print_redirect {
	my $self = shift;
	my ($status, $uri) = @_;

	my $status_line = $STATUS_LINE{$status};
	my ($path, $query, $fragment)
			= $uri =~ /^(.*?)(\?.*?)?(#.*)?$/;
	if ($path !~ m|^/|) {
		$self->{Request}->url =~ m|^(.*/)(?:[^/]*)$|;
		$path = $1 . $path;
	}

	$uri = $self->{Request}->scheme . '://'
		 . $self->{Request}->host
		 . canonpath($path)
		 . (defined $query    ? $query    : '')
		 . (defined $fragment ? $fragment : '');

	print $self->redirect(
				-status	=> "$status $status_line",
				-uri	=> $uri,
			);
	exit;
}

sub print_error {
	my $self = shift;
	my $status = shift;

	my $html = new XiuMai::XHTML($self->{Request});
	$html->accept_language($self->{Request}->accept_language);

	my $status_line = $STATUS_LINE{$status};

	my $method = cdata($self->{Request}->method);
	my $url    = cdata(url_decode($self->{Request}->url));

	my $error_message
		= $status == 400 ? $html->msg('error.400.message') :
		  $status == 401 ? $html->msg('error.401.message') :
		  $status == 403 ? $html->msg('error.403.message', $url) :
		  $status == 404 ? $html->msg('error.404.message', $url) :
		  $status == 405 ? $html->msg('error.405.message', $method, $url) :
		  '';

	$html->charset($self->{CGI}->charset);
	$html->title("$status $status_line");
	my $content = $html->as_string(
			qq(<h1>).cdata($status_line).qq(</h1>),
			qq(<p class="error">).cdata($error_message).qq(</p>),
#			$html->dump(\%ENV, $self),
		);

	print $self->header(
				-status				=> "$status $status_line",
				-content_length		=> length($content),
				-content_language	=> join(', ', $html->content_language),
			);
	print $content						if ($method ne 'HEAD');

	exit;
}

sub print_login_form {
	my $self = shift;
	my ($req) = @_;

	my $html = new XiuMai::XHTML($self->{Request});
	$html->charset($self->{CGI}->charset);
	$html->accept_language($self->{Request}->accept_language);
	$html->title($html->msg('login_form.title'));

	my $title = cdata($html->title);
	my $url   = cdata($self->{Request}->url);

	my $content = $html->as_string(
						qq(<h1><a href="$url">$title</a></h1>),
						$html->login_form($req),
					);

	print $self->header(
				-content_length		=> length($content),
				-content_language	=> join(', ', $html->content_language),
			);
	print $content;
	exit;
}

sub login {
	my $self = shift;
	
	XiuMai::Auth::login(
		$self->{Request}->param('login_name'),
		$self->{Request}->param('passwd'))				or return;

	XiuMai::Auth::cleanup_auth_cookie;

	my $cookie = XiuMai::Auth::make_auth_cookie(
										$self->{Request}->param('login_name'));
	my $path = $self->{Request}->xiumai_base_url;

	$self->{auth_cookie}
		= $self->{CGI}->cookie(
				-name	 => 'XIUMAI_AUTH',
				-value	 => $cookie,
				-path	 => $path,
				-expires => qq(+${XiuMai::Auth::EXPDATE}d),
			);

	$self->print_redirect(301, $self->{Request}->url);
}

sub logout {
	my $self = shift;
	
	my $cookie = $self->{CGI}->cookie('XIUMAI_AUTH')	or return;
	my $path = $self->{Request}->xiumai_base_url;

	$self->{auth_cookie}
		= $self->{CGI}->cookie(
				-name	 => 'XIUMAI_AUTH',
				-value	 => $cookie,
				-path	 => $path,
				-expires => '-1d',
			);

	XiuMai::Auth::delete_auth_cookie($cookie);

	$self->print_redirect(301, $self->{Request}->url);
}

sub print_env {
	my $self = shift;

	my $html = new XiuMai::XHTML($self->{Request});
	$html->charset($self->{CGI}->charset);
	$html->accept_language($self->{Request}->accept_language);
	$html->title($html->msg('getenv.title'));

	my $title = cdata($html->title);
	my $url   = cdata($self->{Request}->url);

	my $content = $html->as_string(
						qq(<h1><a href="$url">$title</a></h1>),
						$html->env,
#						$html->dump($html),
					);

	print $self->header(
				-content_length		=> length($content),
				-content_language	=> join(', ', $html->content_language),
			);
	print $content;
	exit;
}

1;
